// Copyright (C) 2003  Daniel De Graaf
// Main function
// compile with TIGCC 0.94 or later

#include "defines.h"

void _main(void) {
#ifdef Flash
//extfunc as FlashApp is a function and returns true or false
//applications recieve data in non-simplified form; extfunc expects
//simplified data - use push_internal_simplify to do it
//Also, top_estack must remain comstant
//If it is used in an expression {1,extfunc(0),2} then the rest must be
//saved because it is processed by TI, not by extfunc

	ESI saved = top_estack; //save
	ESI rest = saved; 
	while (*rest != END_TAG) rest = next_expression_index(rest);
		//rest is at END_TAG of extfunc arguments
	short templen = (--rest) - bottom_estack;      //length of extra data
	BYTE tempstack[templen+1]; 
	memcpy(tempstack,bottom_estack,templen);       //save temp
	HANDLE h = HS_popEStack();                     //pop estack
	memcpy((char*)bottom_estack,tempstack,templen);//and restore temp
	top_estack = rest;     //restore top_estack so we don't overwrite temp
	ESI z = HToESI(h);     //z is argument pointer
	push_quantum(END_TAG); //an end_tag is nessesary to some functions
	short x = remaining_element_count(z), y; 
	while (x) {            //move all data back to estack, simplified
		ESI t = z;
		for(y = --x; y; y--) //not very efficent, but not many arguments expected
			t = next_expression_index(t); //go back to xth element of popped stack
		push_internal_simplify(t); 
	}
	HeapFree(h);           //free memory of HS_popEStack
	ESI args = top_estack; //set up args to point to the arguments

#else
	ESI args = top_estack; //simple compared to FlashApp code :)
	ESI saved = args; //save
	while (GetArgType(saved) != END_TAG) //code from TIGCC doc
	  saved = next_expression_index (saved);
	saved--;
#endif
	HSym HSfile = HS_NULL;

//Filename
	char *r_name=NULL, *r_str=NULL;
	if (*args == STR_TAG) { //get both SYM_STR and string representations
		r_name = --args;
		while (*(--args));
		r_str = 1 + args--;
	}

//Execute
	HANDLE file = ExecFunc(args); //ExecFunc is assembly

	top_estack = saved;    //restore estack if function messes it up (#31)

	if (!file) {
		push_quantum(FALSE_TAG); //no file made
		return;
	}

//Create file
	if (*r_str == '\\') { //Local variable
	//*(char*)NULL == 0 on TI, so no need to check
		char buffer[16];  //create local variable or store in current folder
		HSfile=SymAdd(FillBufWithLocalName(buffer,r_str,r_name));
	} else if (r_name) HSfile=SymAdd(r_name); //normal variable

	push_quantum(TRUE_TAG); //file made, can corrupt estack now

//attach file to VAT if no errors
	if (HSfile.folder) DerefSym(HSfile)->handle = file;
//otherwise, free space
	else HeapFree(file);
}

/*///////////////////////////
//      Code Examples      //
/////////////////////////////

GetInt     19
GetLastInt 5
OptInt     9
GetStrnArg 15
SymstrArg  4
/////////////Get////////////
Integer = GetInt    (l,u);   - Normal, exits if not on estack
          GetLastInt(l,u);   - does not go to next argument
          OptInt    (l,u,d); - Returns value or defaults to d if not present

String = GetStrnArg(args);

///////////Return///////////
note: EndData is needed to keep TIOS from assuming that IntData is >255.
  For an example: extfunc("x",32,2,"TISLFTST"). Look in VAR-LINK, select
  "x", and press F6: you will see "06-1" - the 06 is an integer that is
  assumed to be > 255

value from 0-65534:
	return OneInt(VALUE);
OR (for lists - see #10 and #19)
	DefData(8);
	IntData(VALUE,0);
	EndData(4);
	return d_h;

true/false:
	DefData(1);
	*data = !!(  ) + FALSE_TAG; // !! makes 1 or 0; construct function to do so
	return d_h;

stirng:
	DefData(len+3);
	*data = 0;
	memcpy(data+1,str,len+1); //or strcpy if zero-terminated
	data[len+2] = STR_TAG;
	return d_h;
	
Lists: first in file <-> last in list
*/


